;;; squeak.el
;;;
;;; Squeak shell for Folktale 0.5.1
;;;
;;; M-x sq - squeak shell
;;; M-p  -- print it (1 line)
;;; M-x sq-file-in-region -- file in
;;;
;;; $Id: squeak.el,v 1.2 2004/05/10 09:47:37 propella Exp $

(global-set-key "\M-p" 'sq-print-line)
(global-set-key "\M-P" 'sq-print-region)

(defvar sq-buffer nil)

;;;; squeak shell
(defun sq-mode ()
  "Squeak shell"
  (interactive)
  (sq-assured-buffer)
  (pop-to-buffer sq-buffer))

(defun sq-print-line ()
  "Eval and print the line"
  (interactive)
  (sq-eval (buffer-substring
			(progn (beginning-of-line) (point))
			(progn (end-of-line) (point)))))

(defun sq-print-region (begin end)
  "Eval and print the region"
  (interactive "r")
  (sq-eval (buffer-substring begin end)))

(defun sq-file-in-region (begin end)
  "File in the region"
  (interactive "r")
  (sq-eval (buffer-substring begin end) t))

(defun sq-connect ()
  (setq sq-buffer (make-comint "squeak" '("localhost" . 4919)))
  (with-current-buffer sq-buffer
	(setq comint-preoutput-filter-functions '(sq-output-filter))))

(defun sq-output-filter (result)
  (unless (eq (current-buffer) sq-buffer)
	(unless (equal result "> ")
	  (insert result)))
  result)

(defun sq-eval (exp &optional chunkp)
  (sq-assured-buffer)
  (let ((send-format (if chunkp
					"%s"
				  "%s!")))
  (with-current-buffer sq-buffer
	(insert "q setChunk")
	(comint-send-input)
	(insert (format send-format exp))
	(comint-send-input)
	(insert "q!")
	(comint-send-input))))

(defun sq-assured-buffer ()
  (cond
   ((null (get-buffer-process nil)) (sq-connect))
   ((eq (process-status sq-buffer) 'open) t)
   (t (progn (delete-buffer sq-buffer) (sq-connect)))))
